#
# Makefile Template for Tessy
# Compiler: Microchip XC8
# Debugger: Microchip MPLAB X
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BUFFER_SIZE            :=
# CHIP                   :=
# COMPILER_DEFINES       :=
# COMPILER_DIALECT       :=
# COMPILER_INCLUDES      :=
# COMPILER_INSTALL_PATH  :=
# ENDIANESS              :=
# EXEFILE_EXTENSION      :=
# LINKER_FILE            :=
# MODULE_PATH            :=
# MODULE_PATH_DOS        :=
# OBJFILE_EXTENSION      :=
# PROJECTROOT            :=
# TARGET_INSTALL_PATH    :=
# TESSY_BIN              :=
# TESSY_SYS              :=
# TESTOBJECT             :=
# USER_LINK_OPTIONS      :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

# Chip          := PIC18F242
TESSY_SYS_DOS   := $(shell ts_pathname -s '$(TESSY_SYS)')
MPLAB_HOME      := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
LINK_FILE_SHORT := $(if $(LINKER_FILE),$(shell ts_pathname -s '$(LINKER_FILE)'))

TS_NAME         := TS_MPLAB
IDE_NAME        := mplabx
COMPANY_NAME    := microchip
PROCESSOR_NAME  := xc8

ifeq ("little","$(ENDIANESS)")
  TS_BYTEORDER := TS_HOST_BYTEORDER
endif
ifeq ("big","$(ENDIANESS)")
  TS_BYTEORDER := TS_NET_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifndef TS_BYTEORDER
  $(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

all : disable_warnings check_success master slave

# Disable special linker warnings.
TS_WARNING_FILE := ts_disable_warnings.h
disable_warnings :
	$(file >$(TS_WARNING_FILE),#pragma warning disable 520,1498,759)

# Cygwin work around.
S_BUILDIN_TESSY_FLAG := $(wildcard $(MODULE_PATH)/tslows_builtin.tessy)

check_success :
ifneq ($(S_BUILDIN_TESSY_FLAG),)
	@echo *
	@echo * Obviously the last build failed. Cleaning up built-in files...
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@echo * Please, try again to generate a built-in target binary.
	@echo *
	@exit 1
endif


#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE     := $(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE     := $(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(COMPANY_NAME)_$(PROCESSOR_NAME).mk


#
#
# SLAVE
#
S_CC            := $(MPLAB_HOME)\bin\xc8-cc.exe
S_LINK          := $(MPLAB_HOME)\bin\xc8-cc.exe

S_INCLUDES      := -I$(TESSY_SYS_DOS)\include\tessy\comm -I$(MODULE_PATH_DOS)
S_STUB_OBJECT   := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT     := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)
S_COMP_OPTIONS  := -mcpu=$(CHIP) -c -x c -D__$(CHIP)__ -D__DEBUG=1  \
                   -Wl,--gc-sections -O1 -ffunction-sections -fdata-sections -fshort-enums -funsigned-char \
				   -funsigned-bitfields -Wall -DXPRJ_default=default -gdwarf-3 -MD -MP $(COMPILER_DIALECT) \
                   -DTESSY -DTS_SLAVE -D$(TS_NAME) -DTS_DISABLE_WARNINGS
S_LINK_OPTIONS  := -mcpu=$(CHIP) -D__DEBUG=1 -Wl,-Map="$(MODULE_PATH_DOS)\ts_$(TESTOBJECT).map" $(if $(MAKEFLAGS),-w) \
                   -DXPRJ_default=default -Wl,--defsym=__MPLAB_BUILD=1 -Wl,--gc-sections \
                   -O0 -O1 -ffunction-sections -fdata-sections -fshort-enums -funsigned-char -funsigned-bitfields \
                   -Wall -gdwarf-3 -Wl,--memorysummary,"$(MODULE_PATH_DOS)\memoryfile.xml" \
				   -Wl,--start-group -Wl,-lm -Wl,--end-group -Wa,-a $(COMPILER_DIALECT) $(LINK_FILE_SHORT)

#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).c
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o "$@" "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

ifneq ($(S_GenerateBuiltinData),)
TARGET_FLAG := d
BUILTIN_OBJ := $(MODULE_PATH_DOS)\tslows_builtin$(OBJFILE_EXTENSION)
master :
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin$(OBJFILE_EXTENSION)
else
BUILTIN_OBJ :=
TARGET_FLAG := s
master : $(MODULE_PATH)/ts_$(TESTOBJECT)_m.exe
endif
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION)
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION) : $(MODULE_PATH)/ts_$(TESTOBJECT)_m.exe

# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" $<

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" $<


#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)


#
#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.c
	@echo *
	@echo ***** Compiling Slave *****
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(S_INCLUDES) $<

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
ifeq ("","$(INITSRCDIR)")
$(info Attribute 'InitSrcDir' is empty. Using objects for linking only.)
START_OBJS := $(wildcard $(INITOBJDIR)/*$(OBJFILE_EXTENSION))
else
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
START_OBJS := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.s,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.s))

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(COMM_INCLUDES) $< -I"$(INITOBJDIR)"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.s
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(COMM_INCLUDES) $< -I"$(INITOBJDIR)"
endif
ifeq ("","$(START_OBJS)")
$(warning Attribute 'Use Board Files' is set to true but no objects were found.)
endif
endif

#
# Objects to be delete after the build process.
#
TS_INTERMEDIATE_OBJS := $(S_OBJECTS) $(BUILTIN_OBJ) $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
                        $(S_SRC_OBJECTS) $(S_UC_OBJECT) $(S_STUB_OBJECT)
.INTERMEDIATE: $(TS_INTERMEDIATE_OBJS)

#
# Linking slave.
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION) : $(START_OBJS) \
                                                                         $(TS_INTERMEDIATE_OBJS)
	@echo *
	@echo ***** Linking Slave *****
	$(S_LINK) $(S_LINK_OPTIONS) $(USER_LINK_OPTIONS) $(START_OBJS) $(TS_INTERMEDIATE_OBJS) -o "$@"
ifneq ($(S_GenerateBuiltinData),)
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@-ts_cp -f "$@" "$(MODULE_PATH)\ts_$(TESTOBJECT)_$(TARGET_FLAG).X\dist\default\production"
	@echo *
	@echo * Generated target binary '$(shell ts_pathname -w $(MODULE_PATH)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION))' containing builtin data.
	@echo *
endif	

#####################################################################
#END_TARGETS
#####################################################################
